program Dixinv_2der

  use sep
  use solver_reg_nrm_mod     !General Norm
  use mask1
  use dix_forward_mod
  use dix_inv_mod
  use dixinv_4der_mod

  implicit none

  real, allocatable               :: vrms(:), vint(:), data(:), model(:), weight(:)
  integer                         :: n1, n2, n3, niter, stat, i, j, k
  real                            :: o1, d1, o2, d2, o3, d3, eps, eps_scale
  real                            :: qntd, qntm
  integer                         :: psiter      !maximum iterations allowed in plan search
  integer                         :: qt_period   !Allow quantile change after certain iterations
  integer                         :: max_overshot_cnt
  character(12)                   :: str_norm_mod, str_norm_dat
  logical, pointer                :: mask(:)

  call sep_init()

  call from_history ("n1", n1)
  call from_history ("o1", o1)
  call from_history ("d1", d1)
  call from_history ("n2", n2)
  call from_history ("o2", o2)
  call from_history ("d2", d2)
  call from_history ("n3", n3, 1)
  call from_history ("o3", o3, 0.0)
  call from_history ("d3", d3, 1.0)

  call to_history ("n1", n1)
  call to_history ("o1", o1)
  call to_history ("d1", d1)
  call to_history ("n2", n2)
  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n3", n3)
  call to_history ("o3", o3)
  call to_history ("d3", d3)
  call to_history ("label1","Time(s)")
  call to_history ("label2","Location(km)")
  call to_history ("label3"," ")

  call from_param ("eps"  , eps  , 100.0)
  call from_param ("eps_scale"  , eps_scale  , 100.0)
  call from_param ("max_overshot_shrink_cnt", max_overshot_cnt, 0)
  call from_param ("qt_period",qt_period,1000)
  call from_param ("niter", niter, 1)
  call from_param ("qntd",qntd,0.8)
  call from_param ("qntm",qntm,0.8)
  call from_param ("psiter",psiter,1)
  call from_param ("norm_dat", str_norm_dat, 'hybrid')
  call from_param ("norm_mod", str_norm_mod, 'hybrid')

  allocate(vrms(n1*n2*n3), vint(n1*n2*n3), data(n1*n2*n3), model(n1*n2*n3), mask(n1*n2*n3), weight(n1*n2*n3))

  call sep_read(vrms)
  call sep_read(weight, "weight")

  do k = 1, n3
    do j = 1, n2
       do i = 1, n1
          data(i+(j-1)*n1+(k-1)*n1*n2) = vrms(i+(j-1)*n1+(k-1)*n1*n2) * vrms(i+(j-1)*n1+(k-1)*n1*n2) * i
       end do
    end do
  end do

  model = 0.0
  mask = .true.
  do k= 1, n3
    do j = 1, n2
       model(1+(j-1)*n1+(k-1)*n1*n2) = 1.5
       mask(1+(j-1)*n1+(k-1)*n1*n2) = .false.
!       model(n1+(j-1)*n1+(k-1)*n1*n2) = 3.0
!       mask(n1+(j-1)*n1+(k-1)*n1*n2) = .false.
    end do
  end do

  call wt_init(n1, n2, n3, weight)
  call dix_fw_init(n1, n2, n3)
  call dix_reg_init(n1, n2, n3)
  call mask1_init(mask)

  eps = eps/eps_scale

  call solver_reg_nrm(m=model, d=data, Fop=dix_fw_lop, Aop=dix_reg_lop, Wop=wt_dix, Jop=mask1_lop, m0=model, stepper=cgstep_ps, nAop=size(model)*4, niter=niter, eps=eps, norm_dat_in=str_norm_dat, norm_mod_in=str_norm_mod, qnt_dat_in=qntd, qnt_mod_in=qntm, qt_period_in=qt_period, psiter=psiter, overshot_shrink_max_cnt_in=max_overshot_cnt)

  call solver_reg_nrm_close()

  do k = 1, n3
    do j = 1, n2
      do i = 1, n1
        if (model(i+(j-1)*n1+(k-1)*n1*n2)<0.0) then
          vint(i+(j-1)*n1+(k-1)*n1*n2) = -sqrt(abs(model(i+(j-1)*n1+(k-1)*n1*n2)))
        else
          vint(i+(j-1)*n1+(k-1)*n1*n2) = sqrt(model(i+(j-1)*n1+(k-1)*n1*n2))
        end if
      end do
    end do
  end do

  call sep_write(vint)

  if (exist_file("recons")) then
    write(0,*) "output recon"
    call to_history("n1",n1,"recons")
    call to_history("o1",o1,"recons")
    call to_history("d1",d1,"recons")
    call to_history("n2",n2,"recons")
    call to_history("o2",o2,"recons")
    call to_history("d2",d2,"recons")
    call to_history("n3",n3,"recons")
    call to_history("o3",o3,"recons")
    call to_history("d3",d3,"recons")
    call to_history  ("label1","time(s)","recons")
    call to_history  ("label2","location(m)",'recons')
    call to_history  ("label3","vel(m/s)",'recons')
    stat = dix_fw_lop(.false.,.false.,model,data)
    do k = 1, n3
      do j = 1, n2
        do i = 1, n1
          if (data(i+(j-1)*n1+(k-1)*n1*n2)<0.0) then
            vrms(i+(j-1)*n1+(k-1)*n1*n2) = -sqrt(abs(data(i+(j-1)*n1+(k-1)*n1*n2)/i))
          else
            vrms(i+(j-1)*n1+(k-1)*n1*n2) = sqrt(data(i+(j-1)*n1+(k-1)*n1*n2)/i)
          end if
        end do
      end do
    end do
    call sep_write(vrms,"recons")
  end if

  deallocate(vrms, vint, data, model, mask)
  call sep_close()
  call wt_dix_close()
  call dix_fw_close()

end program
