module rms2int_mod

implicit none

contains

subroutine rms2int(  inverse, dt, vrms, nt,  vint )
integer it, wide,    inverse,           nt
real    vmin,                 dt, vrms( nt), vint( nt)
real                     sum( nt), vis ( nt)
if ( inverse .eq. 0) then
  do it= 1, nt
    sum(it)= ((it-1)*dt) * amax1( vrms(it)**2, 1.5**2 )
  end do 
  vis(1) = amin1( vrms(1) ** 2 , 1.5 ** 2 )
  do it= 2, nt
    vis(it) = ( sum(it) - sum(it-1) )/ dt
  end do 
  wide= 2
  do   
    vmin = vis(1)
    do it=1,nt  
      if ( vis(it)<vmin) then
        vmin = vis(it) 
      end if
    end do 
    if ( vmin > 0 ) then
      exit
    end if
    call triangle( wide, 1, nt, vis, vis)           ! smooth vis()
    wide = wide + 1
    if ( wide >= nt/3) then
      call erexit('negative velocity')
    end if
  end do 
  do it= 1, nt
    vint(it) = sqrt( vis(it))
  end do
else
  do it= 1, nt
    vis(it) = vint(it) ** 2
  end do 
  sum(1) = 0.
  do it= 2, nt
    sum(it) = sum(it-1) + vis(it) * dt
  end do 
  vrms(1) = vint(1)
  do it= 2, nt
    vrms(it) = sqrt( sum(it) / ((it-1)*dt) )
  end do
end if 

end subroutine


subroutine triangle( nr, m1, n12, uu, vv)
! input:        nr      rectangle width (points) (Triangle base twice as wide.)
! input:        uu(m1,i2),i2=1,n12      is a vector of data.
! output:       vv(m1,i2),i2=1,n12      may be on top of uu
integer nr,m1,n12, i,np,nq
real uu( m1, n12),  vv( m1, n12)        
real pp(n12+nr-1), qq(n12+nr+nr-2), tt(n12)
do i=1,n12  
  qq(i) = uu(1,i)
end do 
if ( n12 .eq. 1 ) then
  call copy( n12, qq, tt)
else
  call boxconv( nr, n12, qq, pp)
  np = nr+n12-1
  call boxconv( nr, np , pp, qq)
  nq = nr+np-1
  do i= 1, n12
    tt(i) = qq(i+nr-1)
  end do 
  do i= 1, nr-1                                   
  ! fold back near end
    tt(i) = tt(i) + qq(nr-i)
  end do 
  do i= 1, nr-1                                   ! fold back far end
    tt(n12-i+1) = tt(n12-i+1) + qq(n12+(nr-1)+i)
  end do
end if 
do i=1,n12  
  vv(1,i) = tt(i)
end do 

end subroutine


subroutine boxconv( nb, nx, xx, yy)
! inputs:       nx,  xx(i), i=1,nx      the data
!               nb                      the box length
! output:       yy(i),i=1,nx+nb-1       smoothed data
integer nx, ny, nb, i
real xx(nx), yy(1)
real bb(nx+nb)
if ( nb < 1 .or. nb > nx) then
  call erexit('boxconv')  ! "||" means .OR.
end if
ny = nx+nb-1
do i= 1, ny
  bb(i) = 0.
end do 
bb(1) = xx(1)
do i= 2, nx
  bb(i) = bb(i-1) + xx(i)         ! make B(Z) = X(Z)/(1-Z)
end do 
do i= nx+1, ny
  bb(i) = bb(i-1)
end do 
do i= 1, nb
  yy(i) = bb(i)
end do 
do i= nb+1, ny
  yy(i) = bb(i) - bb(i-nb)        ! make Y(Z) = B(Z)*(1-Z**nb)
end do 
do i= 1, ny
  yy(i) = yy(i) / nb
end do 

end subroutine


subroutine copy( n, xx, yy)
integer i, n
real xx(n), yy(n)
do i= 1, n
  yy(i) = xx(i)
end do 

end subroutine

end module
